// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

var GM_gOverlayTooltip = {
  _account: null,
  
  populate: function()
  {
    this._account = GM_gOverlay._accounts[GM_gOverlay._current];
    
    // Hide the tooltip
    GM_getEBI("gm-toolbar-tooltip-header").hidden = "true";
    GM_getEBI("gm-toolbar-tooltip-msg").hidden = "true";
    GM_getEBI("gm-toolbar-tooltip-details").hidden = "true";
    
    if (this._account == null)
      this._setMsg("click-add-account");
    else
    {
      this._setHeader("tb-tooltip-logged-out");
      
      switch (this._account.status)
      {
        case GM_NOTIFY_ACCOUNT_STATE_ERROR_PASSWORD:
          this._setMsg("tb-tooltip-msg-error-password");
          break;
        case GM_NOTIFY_ACCOUNT_STATE_ERROR_CAPTCHA:
          this._setMsg("tb-tooltip-msg-error-captcha");
          break;
        case GM_NOTIFY_ACCOUNT_STATE_ERROR_NETWORK:
          this._setMsg("tb-tooltip-msg-error-network");
          break;
        case GM_NOTIFY_ACCOUNT_STATE_ERROR_SERVER:
          this._setMsg("tb-tooltip-msg-error-server");
          break;
        case GM_NOTIFY_ACCOUNT_STATE_ERROR_TIMEOUT:
          this._setMsg("tb-tooltip-msg-error-timeout");
          break;
        case GM_NOTIFY_ACCOUNT_STATE_CONNECTING:
        {
          this._setHeader("tb-tooltip-connecting");
          
          if (this._account.loggedIn)
            this._setDetails();
          else
            this._setMsg("tb-tooltip-msg-connecting");
          
          break;
        }
        case GM_NOTIFY_ACCOUNT_STATE_LOGGED_OUT:
        case GM_NOTIFY_ACCOUNT_STATE_LOGGED_OUT_NO_PASSWORD:
          this._setMsg("tb-tooltip-msg-logged-out");
          break;
        case GM_NOTIFY_ACCOUNT_STATE_LOGGED_IN_NO_MAIL:
        case GM_NOTIFY_ACCOUNT_STATE_LOGGED_IN_NEW_MAIL:
          this._setHeader("tb-tooltip-logged-in");
          this._setDetails();
          break;
      }
    }
  },
  
  _setHeader: function(aHeader)
  {
    var parent = GM_getEBI("gm-toolbar-tooltip-header");
    var item = null;
    
    // Remove all labels
    while (parent.hasChildNodes())
      parent.removeChild(parent.lastChild);
    
    item = document.createElement("image");
    item.setAttribute("class", "gm-icon");
    item.setAttribute("iconsize", "small");
    item.setAttribute("status", this._account.status);
    item.setAttribute("newMail", (GM_gOverlay._getUnread(this._account.email) > 0 ? "true" : "false"));
    item.setAttribute("style", "margin-right:6px;");
    parent.appendChild(item);
    
    item = document.createElement("label");
    item.setAttribute("value", this._account.alias);
    item.setAttribute("style", "font-size: 12px; font-weight: bold; margin: 0px;");
    parent.appendChild(item);
    
    item = document.createElement("label");
    item.setAttribute("value", GM_getBundleString(aHeader));
    item.setAttribute("style", "font-size: 12px; margin: 0 0 0 4px;");
    parent.appendChild(item);
    
    parent.setAttribute("align", "center");
    parent.setAttribute("hidden", "false");
  },
  
  _setMsg: function(aMsg)
  {
    var parent = GM_getEBI("gm-toolbar-tooltip-msg");
    var msg = GM_getBundleString(aMsg).split("|");
    var item = null;
    
    // Remove all labels
    while (parent.hasChildNodes())
      parent.removeChild(parent.lastChild);
    
    for (var i = 0; i < msg.length; i++)
    {
      item = document.createElement("label");
      item.setAttribute("value", msg[i]);
      item.setAttribute("style", "margin:2px");
      parent.appendChild(item);
    }
    
    parent.setAttribute("hidden", "false");
  },
  
  _setDetails: function()
  {
    var parent = null;
    var item = null;
    var unread = null;
    
    // Inbox
    unread = this._account.inboxUnread;
    item = GM_getEBI("gm-toolbar-tooltip-details-inbox");
    item.setAttribute("value", GM_getBundleFString("tb-tooltip-details-inbox", [unread, GM_getBundleString((unread == 1 ? "msg" : "msgs"))]));
    
    // Drafts
    unread = this._account.savedDrafts;
    item = GM_getEBI("gm-toolbar-tooltip-details-drafts");
    item.setAttribute("value", GM_getBundleFString("tb-tooltip-details-drafts", [unread, GM_getBundleString((unread == 1 ? "draft" : "drafts"))]));
    item.setAttribute("hidden", (unread == 0));
    
    // Spam
    unread = this._account.spamUnread;
    item = GM_getEBI("gm-toolbar-tooltip-details-spam");
    item.setAttribute("value", GM_getBundleFString("tb-tooltip-details-spam", [unread, GM_getBundleString((unread == 1 ? "msg" : "msgs"))]));
    item.setAttribute("hidden", (unread == 0));
    
    // Labels
    var parent = GM_getEBI("gm-toolbar-tooltip-details-labels");
    var labels = this._account.getLabels({});
    
    while (parent.hasChildNodes())
      parent.removeChild(parent.lastChild);
    
    for (var i = 0; i < labels.length; i++)
    {
      var row = document.createElement("row");
      row.setAttribute("align", "center");
      
      if (labels[i].unread > 0)
        row.setAttribute("style", "font-weight:bold;");
      
      item = document.createElement("label");
      item.setAttribute("value", GM_unicode(labels[i].name));
      row.appendChild(item);
      
      item = document.createElement("label");
      item.setAttribute("value", labels[i].unread);
      item.setAttribute("flex", "1");
      row.appendChild(item);
      
      if (labels[i].unread > 0 || !this._account.prefs.getBoolValue("toolbar-tooltip-show-labels"))
        parent.appendChild(row);
    }
    
    parent.parentNode.setAttribute("collapsed", !parent.hasChildNodes());
    parent.parentNode.setAttribute("hidden", !parent.hasChildNodes());
    
    // Space
    item = GM_getEBI("gm-toolbar-tooltip-details-space");
    item.setAttribute("value", GM_getBundleFString("tb-tooltip-details-space",
                      [this._account.spaceUsed, this._account.percentUsed, this._account.totalSpace]));
    
    // Snippets
    var parent = GM_getEBI("gm-toolbar-tooltip-details-snippets");
    var sep = GM_getEBI("gm-toolbar-tooltip-details-snippets-sep");
    var snippets = this._account.getSnippets({});
    
    while (parent.hasChildNodes())
      parent.removeChild(parent.lastChild);
    
    if (this._account.prefs.getBoolValue("toolbar-tooltip-show-snippets"))
    {
      for (var i = 0; i < snippets.length && i < 10; i++)
      {
        //var eastern = new Date("Mon Apr 3 2006 10:39 PM");
        //alert(eastern.toLocaleTimeString());
        
        item = document.createElement("label");
        item.setAttribute("value", (i+1) + ". " + snippets[i].time + " - " + GM_unicode(snippets[i].sender) + " - " + GM_unicode(snippets[i].subject));
        item.setAttribute("crop", "end");
        item.setAttribute("style", "margin-top:8px; font-size:10px; font-weight:bold;");
        parent.appendChild(item);
        
        item = document.createElement("description");
        item.setAttribute("value", GM_unicode(snippets[i].msg));
        item.setAttribute("crop", "end");
        item.setAttribute("style", "font-size:9px;");
        parent.appendChild(item);
      }
    }
    
    sep.setAttribute("hidden", !parent.hasChildNodes());
    parent.setAttribute("hidden", !parent.hasChildNodes());
    
    GM_setEBIA("gm-toolbar-tooltip-details", "hidden", "false");
  }
}