// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const GM_CC = Components.classes;
const GM_CI = Components.interfaces;

const GM_GMAIL_ICON = "chrome://gmanager/skin/gm-icon.png";
const GM_PROPERTIES = "chrome://gmanager/locale/gmanager.properties";
const GM_WEBSITE = "http://www.longfocus.com/firefox/gmanager/";

const GM_NOTIFY_ACCOUNT_CHANGE = "gm-account-change";
const GM_NOTIFY_ACCOUNT_LOGIN = "gm-account-login";
const GM_NOTIFY_ACCOUNT_PREFS_CHANGE = "gm-account-prefs-change";
const GM_NOTIFY_ACCOUNT_RESET_UNREAD = "gm-reset-unread-count";
const GM_NOTIFY_ACCOUNT_ALERT_FINISHED = "gm-alert-finished";
const GM_NOTIFY_ACCOUNT_ALERT_LINK_CLICK = "gm-alert-link-click";

const GM_NOTIFY_ACCOUNT_STATE_CONNECTING = "gm-account-state-connecting";
const GM_NOTIFY_ACCOUNT_STATE_LOGGED_OUT = "gm-account-state-logged-out";
const GM_NOTIFY_ACCOUNT_STATE_LOGGED_OUT_NO_PASSWORD = "gm-account-state-logged-out-no-password";
const GM_NOTIFY_ACCOUNT_STATE_LOGGED_IN_NO_MAIL = "gm-account-state-logged-in-no-mail";
const GM_NOTIFY_ACCOUNT_STATE_LOGGED_IN_NEW_MAIL = "gm-account-state-logged-in-new-mail";
const GM_NOTIFY_ACCOUNT_STATE_ERROR_PASSWORD = "gm-account-state-error-password";
const GM_NOTIFY_ACCOUNT_STATE_ERROR_CAPTCHA = "gm-account-state-error-captcha";
const GM_NOTIFY_ACCOUNT_STATE_ERROR_NETWORK = "gm-account-state-error-network";
const GM_NOTIFY_ACCOUNT_STATE_ERROR_SERVER = "gm-account-state-error-server";
const GM_NOTIFY_ACCOUNT_STATE_ERROR_TIMEOUT = "gm-account-state-error-timeout";

/**
 * Document element
 */
function GM_getEBI(aId) {
  return document.getElementById(aId);
}

function GM_setEBIA(aId, aAttr, aValue) {
  GM_getEBI(aId).setAttribute(aAttr, aValue);
}

/**
 * String bundle
 */
function GM_getBundle() {
  return GM_CC["@mozilla.org/intl/stringbundle;1"]
           .getService(GM_CI.nsIStringBundleService)
           .createBundle(GM_PROPERTIES);
}

function GM_getBundleString(aName) {
  return GM_getBundle().GetStringFromName(aName);
}

function GM_getBundleFString(aName, aParams) {
  return GM_getBundle().formatStringFromName(aName, aParams, aParams.length);
}

/**
 * Prompt service
 */
/*
function GM_confirm(aMsg) {
  return GM_CC["@mozilla.org/embedcomp/prompt-service;1"]
           .getService(GM_CI.nsIPromptService)
           .confirm(window, GM_getBundleString("gm-ext-name"), aMsg);
}
*/

/**
 * Logging service
 */
function GM_log(aStr) {
  var console = GM_CC['@mozilla.org/consoleservice;1'].getService(GM_CI.nsIConsoleService);
  console.logStringMessage(aStr);
}

/**
 * Manager utils
 */
function GM_getManager() {
  return GM_CC["@longfocus.com/gmanager/manager;1"].getService(GM_CI.gmIManager);
}

function GM_getBranch() {
  return GM_getManager().branch;
}

function GM_getGlobal() {
  return GM_getManager().getAccount("global").prefs;
}

function GM_getCurrent() {
  var current = GM_getBranch().getCharPref("current");
  return (current == "" ? null : current);
}

function GM_queryService(aAccount) {
  switch (aAccount.type)
  {
    case "gmail":
      // Gmail service
      aAccount.QueryInterface(GM_CI.gmIServiceGmail);
      break;
    default:
      break;
  }
}

/**
 * Misc utils
 */
function GM_unicode(aText) {
  var conv = GM_CC["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(GM_CI.nsIScriptableUnicodeConverter);
  conv.charset = "UTF-8";
  return conv.ConvertToUnicode(aText);
}

function GM_getBrowser() {
  return GM_CC["@mozilla.org/appshell/window-mediator;1"]
           .getService(GM_CI.nsIWindowMediator)
           .getMostRecentWindow("navigator:browser");
}

function GM_getNumContextMenu() {
  var num = null;
  if (GM_getBrowser())
    num = GM_getBrowser().GM_getEBI("contentAreaContextMenu").childNodes.length;
  return num;
}

function GM_getNumStatusbar() {
  var num = null;
  if (GM_getBrowser())
    num = GM_getBrowser().GM_getEBI("status-bar").childNodes.length;
  return num;
}

function GM_getMyVersion() {
  return GM_getManager().version;
}

function GM_visitSite(aSite) {
  var ioService = GM_CC["@mozilla.org/network/io-service;1"].createInstance(GM_CI.nsIIOService);
  var refString = GM_WEBSITE + "v/" + GM_getMyVersion() + "/";
  var refUri = ioService.newURI(refString, null, null);
  
  GM_getBrowser().GM_getEBI("content").addTab(aSite, refUri);
}

/**
 * Load Default Mail Client
 */
function GM_loadDefaultMail(aCompose) {
  var mailtoUrl = "mailto:";
  
  if (aCompose != null)
    mailtoUrl = aCompose.orig;
  
  var extProtocolSvc = GM_CC["@mozilla.org/uriloader/external-protocol-service;1"].getService(GM_CI.nsIExternalProtocolService);
  var ioService = GM_CC["@mozilla.org/network/io-service;1"].getService(GM_CI.nsIIOService);
  var uri = ioService.newURI(mailtoUrl, null, null);
  
  if (extProtocolSvc)
    extProtocolSvc.loadUrl(uri);
}

/**
 * Load Gmail Page
 */
function GM_loadGmailPage(aType, aAccount, aLoc, aCompose) {
  var email = aAccount.email;
  var scheme = (aAccount.prefs.getBoolValue("general-secured-connection") ? "https" : "http");
  var isHosted = false;
  var url = "https://www.google.com/accounts/ServiceLoginAuth";
  var username = null;
  var domain = null;
  var loc = aLoc;
  var data = null;
  
  if (email.indexOf("@gmail.com") == -1 && email.indexOf("@googlemail.com") == -1)
  {
    isHosted = true;
    username = email.split("@")[0];
    domain = email.split("@")[1];
    url = "https://www.google.com/a/" + domain + "/LoginAction";
  }
  
  switch (aType)
  {
    case "inbox":
    {
      // Reset unread count
      if (aAccount.loggedIn && GM_getGlobal().getBoolValue("toolbar-reset-unread-count"))
        GM_gOverlay._observer.notifyObservers(null, GM_NOTIFY_ACCOUNT_RESET_UNREAD, null);
      
      if (!isHosted)
        data = "ltmpl=wsad&ltmplcache=2" +
               "&continue=" + scheme + "://mail.google.com/mail/&service=mail&rm=false&ltmpl=wsad" +
               "&Email=" + encodeURIComponent(aAccount.email) +
               "&Passwd=" + encodeURIComponent(aAccount.password) +
               "&null=Sign+in";
      else
        data = "at=null" + 
               "&continue=" + scheme + "://mail.google.com/hosted/" + domain + "/&service=mail" +
               "&userName=" + encodeURIComponent(username) +
               "&password=" + encodeURIComponent(aAccount.password);
      
      break;
    }
    case "compose":
    {
      var compose = (aCompose != null ? aCompose.fmtd : "");
      
      if (loc == null)
        loc = GM_getGlobal().getIntValue("compose-tab-location");
      
      if (!isHosted)
        data = "ltmpl=wsad&ltmplcache=2" + 
               "&continue=" + scheme +
               encodeURIComponent("://mail.google.com/mail/?view=cm&fs=1" + compose) +
               "&service=mail&rm=false&ltmpl=wsad" +
               "&Email=" + encodeURIComponent(aAccount.email) + 
               "&Passwd=" + encodeURIComponent(aAccount.password) + 
               "&null=Sign+in";
      else
      {
        data = "at=null" + 
               "&continue=" + scheme + "://mail.google.com/hosted/" + domain + "/&service=mail" +
               "&userName=" + encodeURIComponent(username) +
               "&password=" + encodeURIComponent(aAccount.password);
        
        /*
        data = "at=null" + 
               "&continue=" + scheme +
               encodeURIComponent("://mail.google.com/hosted/" + domain + "/?&view=cm&fs=1" + compose) +
               "&userName=" + encodeURIComponent(username) +
               "&password=" + encodeURIComponent(aAccount.password);
        */
      }
      
      break;
    }
  }
  
  // Loads the data
  GM_loadURI.load(url, data, loc);
}

/**
 * Load URI
 */
var GM_loadURI = {
  _observer: GM_CC["@mozilla.org/observer-service;1"].getService(GM_CI.nsIObserverService),
  _browser: GM_getBrowser().getBrowser(),
  _flags: GM_CI.nsIWebNavigation.LOAD_FLAGS_NONE,
  _url: null,
  _mimeStream: null,
  
  load: function(aURL, aData, aLocation)
  {
    // Sets the post data
    var postData = GM_CC["@mozilla.org/io/string-input-stream;1"].createInstance(GM_CI.nsIStringInputStream);
    postData.setData(aData, aData.length);
    
    // Sets the url
    this._url = aURL;
    
    // Initializes the mime stream
    this._mimeStream = GM_CC["@mozilla.org/network/mime-input-stream;1"].createInstance(GM_CI.nsIMIMEInputStream);
    this._mimeStream.addHeader("Content-Type", "application/x-www-form-urlencoded");
    this._mimeStream.addHeader("Content-Length", aData.length);
    this._mimeStream.setData(postData);
    
    switch (aLocation)
    {
      case 0: // Blank tab (first available)
      {
        var browser = null;
        
        for (var i = 0; i < this._browser.browsers.length && browser == null; i++)
          if (this._browser.browsers.item(i).currentURI.asciiHost == "")
            browser = this._browser.browsers.item(i);
        
        if (browser != null)
          browser.webNavigation.loadURI(aURL, this._flags, null, this._mimeStream, null);
        else
          this.load(aURL, aData, 3);
        
        break;
      }
      case 1: // Current tab
      {
        this._browser.webNavigation.loadURI(aURL, this._flags, null, this._mimeStream, null);
        break;
      }
      case 2: // New tab (focused)
      {
        this._browser.selectedTab = this._browser.addTab();
        this._browser.webNavigation.loadURI(aURL, this._flags, null, this._mimeStream, null);
        break;
      }
      case 3: // New tab (in background)
      {
        var newTab = this._browser.getBrowserForTab(this._browser.addTab());
        newTab.webNavigation.loadURI(aURL, this._flags, null, this._mimeStream, null);
        break;
      }
      case 4: // New Window
      {
        this._observer.addObserver(this, "browser-window-before-show", false);
        window.open();
        break;
      }
    }
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic == "browser-window-before-show" && this._mimeStream != null)
    {
      // Remove observer
      this._observer.removeObserver(this, "browser-window-before-show");
      
      // Loads page in browser
      GM_getBrowser().getBrowser().webNavigation.loadURI(this._url, this._flags, null, this._mimeStream, null);
      
      // Reset data
      this._url = null;
      this._mimeStream = null;
    }
  }
}

/**
 * Sounds
 */
var GM_gSound = {
  select: function()
  {
    var file = null;
    var nsIFilePicker = GM_CI.nsIFilePicker;
    var filePicker = GM_CC["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    
    filePicker.init(window, GM_getBundleString("prefs-select-sound"), nsIFilePicker.modeOpen);
    filePicker.appendFilter(GM_getBundleString("prefs-sound-files"), "*.wav");
    filePicker.appendFilters(nsIFilePicker.filterAll);
    
    if (filePicker.show() == nsIFilePicker.returnOK)
      file = filePicker.file.path;
    
    return file;
  },
  
  preview: function(aFile)
  {
    var url = null;
    var soundService = GM_CC["@mozilla.org/sound;1"].createInstance(GM_CI.nsISound);
    var ioService = GM_CC["@mozilla.org/network/io-service;1"].getService(GM_CI.nsIIOService);
    
    try {
      if (aFile.indexOf("file://") == -1) {
        var localFile = GM_CC["@mozilla.org/file/local;1"].createInstance(GM_CI.nsILocalFile);
        localFile.initWithPath(aFile);
        url = ioService.newFileURI(localFile, null, null);
      } else {
        url = ioService.newURI(aFile, null, null);
      }
      
      soundService.play(url);
    } catch (e) {}
  }
}

/**
 * Alerts
 */
var GM_gAlerts = {
  _accountQueue: new Array(),
  _current: null,
  _displaying: false,
  
  show: function(aAccount)
  {
    if (this._accountQueue == null)
      this._accountQueue = new Array();
    
    if (aAccount != null)
    {
      var exists = false;
      
      if (this._current != null)
        exists = (aAccount.email == this._current.email);
      
      for (var i = 0; i < this._accountQueue.length && !exists; i++)
        if (aAccount.email == this._accountQueue[i].email)
          exists = true;
      
      if (!exists)
        this._accountQueue.push(aAccount);
    }
    
    if (!this._displaying && this._accountQueue.length > 0)
    {
      this._current = this._accountQueue.shift();
      
      if (this._current.loggedIn && this._current.getSnippets({}).length != 0)
      {
        var clickable = GM_getGlobal().getBoolValue("notifications-clickable-alerts");
        
        // Display the snippets
        window.openDialog("chrome://gmanager/content/alert.xul", "alerts", "chrome,dialog=yes,titlebar=no,popup=yes", this._current, clickable, this);
        
        // Set displaying
        this._displaying = true;
      }
      else
        this.show(null);
    }
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    switch (aTopic)
    {
      case GM_NOTIFY_ACCOUNT_ALERT_FINISHED:
      {
        // Set displaying
        this._displaying = false;
        
        // Checks account queue
        if (this._accountQueue.length > 0)
        {
          // Shows next account
          this.show(null);
        }
        else
        {
          // Clear alert data
          this._accountQueue = null;
          this._current = null;
        }
        
        break;
      }
      case GM_NOTIFY_ACCOUNT_ALERT_LINK_CLICK:
      {
        // Loads the page (new tab in background)
        GM_loadGmailPage("inbox", aData, 3, null);
        
        break;
      }
      default:
        break;
    }
  }
}

/**
 * Version 0.5 preferences migration
 */
function GM_migratePrefs() {
  var observer = GM_CC["@mozilla.org/observer-service;1"].getService(GM_CI.nsIObserverService);
  var branch = GM_CC["@mozilla.org/preferences-service;1"].getService(GM_CI.nsIPrefService).getBranch("gmanager.");
  var accounts = null;
  var current = null;
  
  try {
    accounts = branch.getCharPref("accounts").split(",");
    current = branch.getCharPref("current").split(",")[0];
  } catch(e) {}
  
  if (accounts == null)
    return;
  
  for (var i = 0; i < accounts.length; i++)
  {
    var username = accounts[i];
    var email = username + "@gmail.com";
    var password = null;
    var remember = false;
    
    try {
      var host = {value:""};
      var user = {value:""};
      var pass = {value:""};
      
      var pmi = GM_CC["@mozilla.org/passwordmanager;1"].getService(GM_CI.nsIPasswordManagerInternal);
      pmi.findPasswordEntry("gmanager.gmail.account", username, "", host, user, pass);
      
      password = pass.value;
      remember = true;
    } catch(e) {}
    
    // Add account
    GM_getManager().addAccount("gmail", email, password, remember, email);
  }
  
  if (accounts.length > 0)
  {
    // Notify account change
    observer.notifyObservers(null, GM_NOTIFY_ACCOUNT_CHANGE, "migrate");
    
    // Set current account
    if (current != null)
      GM_getBranch().setCharPref("current", current + "@gmail.com");
  }
}